"""All Dataset related functions
"""
from ..base import Base
from .documents import Documents
from .monitor import Monitor
from .tasks import Tasks


class Datasets(Base):
    """All dataset-related functions"""

    def __init__(self, project: str, api_key: str, base_url: str):
        self.base_url = base_url
        self.project = project
        self.api_key = api_key
        self.tasks = Tasks(project=project, api_key=api_key, base_url=base_url)
        self.documents = Documents(project=project, api_key=api_key, base_url=base_url)
        self.monitor = Monitor(project=project, api_key=api_key, base_url=base_url)

        super().__init__(project, api_key, base_url)

    def test(self):
        return self.wtf

    def schema(
        self, dataset_id: str, output_format: str = "json", verbose: bool = True
    ):
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/schema",
            method="GET",
            output_format=output_format,
            verbose=verbose,
        )

    def metadata(
        self, dataset_id: str, output_format: str = "json", verbose: bool = True
    ):
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/metadata",
            method="GET",
            output_format=output_format,
            verbose=verbose,
        )

    def stats(self, dataset_id: str, output_format: str = "json", verbose: bool = True):
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/monitor/stats",
            method="GET",
            output_format=output_format,
            verbose=verbose,
        )

    def health(
        self, dataset_id: str, output_format: str = "json", verbose: bool = True
    ):
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/monitor/health",
            method="GET",
            output_format=output_format,
            verbose=verbose,
        )

    def create(
        self,
        dataset_id: str,
        schema: dict = {},
        output_format: str = "json",
        verbose: bool = True,
    ):
        return self.make_http_request(
            endpoint=f"datasets/create",
            method="POST",
            parameters={"id": dataset_id, "schema": schema},
            output_format=output_format,
            verbose=verbose,
        )

    def list(self, output_format: str = "json", verbose: bool = True, retries=None):
        return self.make_http_request(
            endpoint="datasets/list",
            method="GET",
            output_format=output_format,
            verbose=verbose,
            retries=retries,
        )

    def list_all(
        self,
        include_schema: bool = True,
        include_stats: bool = True,
        include_metadata: bool = True,
        include_schema_stats: bool = False,
        include_vector_health: bool = False,
        include_active_jobs: bool = False,
        dataset_ids: list = [],
        sort_by_created_at_date: bool = False,
        asc: bool = False,
        page_size: int = 20,
        page: int = 1,
        output_format: str = "json",
        verbose: bool = True,
    ):
        return self.make_http_request(
            endpoint="datasets/list",
            method="POST",
            parameters={
                "include_schema": include_schema,
                "include_stats": include_stats,
                "include_metadata": include_metadata,
                "include_schema_stats": include_schema_stats,
                "include_vector_health": include_vector_health,
                "include_active_jobs": include_active_jobs,
                "dataset_ids": dataset_ids,
                "sort_by_created_at_date": sort_by_created_at_date,
                "asc": asc,
                "page_size": page_size,
                "page": page,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def facets(
        self,
        dataset_id,
        fields: list = [],
        date_interval: str = "monthly",
        page_size: int = 5,
        page: int = 1,
        asc: bool = False,
        output_format: str = "json",
        verbose: bool = True,
    ):
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/facets",
            method="POST",
            parameters={
                "fields": fields,
                "date_interval": date_interval,
                "page_size": page_size,
                "page": page,
                "asc": asc,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def check_missing_ids(
        self, dataset_id, ids, output_format: str = "json", verbose: bool = True
    ):

        # Check if dataset_id exists
        dataset_exists = dataset_id in self.list()["datasets"]

        if dataset_exists:
            return self.make_http_request(
                endpoint=f"datasets/{dataset_id}/documents/get_missing",
                method="GET",
                parameters={"ids": ids},
                output_format=output_format,
                verbose=verbose,
            )

        else:
            print("Dataset does not exist")
            return

    def bulk_insert(
        self,
        dataset_id: str,
        documents: list,
        insert_date: bool = True,
        overwrite: bool = True,
        update_schema: bool = True,
        field_transformers=[],
        verbose: bool = True,
        return_documents: bool = False,
        retries: int = None,
        output_format: str = "json",
        base_url="https://ingest-api-dev-aueast.relevance.ai/latest/",
    ):
        """
        Documentation can be found here: https://ingest-api-dev-aueast.relevance.ai/latest/documentation#operation/InsertEncode
        An example field_transformers object:
        {
            "field": "string",
            "output_field": "string",
            "remove_html": true,
            "split_sentences": true
        }
        """
        if return_documents is False:
            return self.make_http_request(
                endpoint=f"datasets/{dataset_id}/documents/bulk_insert",
                base_url=base_url,
                method="POST",
                parameters={
                    "documents": documents,
                    "insert_date": insert_date,
                    "overwrite": overwrite,
                    "update_schema": update_schema,
                    "field_transformers": field_transformers,
                },
                output_format=output_format,
                retries=retries,
                verbose=verbose,
            )

        else:
            insert_response = self.make_http_request(
                endpoint=f"datasets/{dataset_id}/documents/bulk_insert",
                base_url=base_url,
                method="POST",
                parameters={
                    "documents": documents,
                    "insert_date": insert_date,
                    "overwrite": overwrite,
                    "update_schema": update_schema,
                    "field_transformers": field_transformers,
                },
                output_format=False,
                retries=retries,
                verbose=verbose,
            )

            try:
                response_json = insert_response.json()
            except:
                response_json = None

            return {
                "response_json": response_json,
                "documents": documents,
                "status_code": insert_response.status_code,
            }

    def delete(
        self,
        dataset_id: str,
        confirm: bool = False,
        output_format: str = "json",
        verbose: bool = True,
    ):
        if confirm:
            # confirm with the user
            self.logger.critical(f"You are about to delete {dataset_id}")
            user_input = input("Confirm? [Y/N] ")
        else:
            user_input = "y"
        # input validation
        if user_input.lower() in ("y", "yes"):
            return self.make_http_request(
                endpoint=f"datasets/delete",
                method="POST",
                parameters={"dataset_id": dataset_id},
                output_format=output_format,
                verbose=verbose,
            )

        elif user_input.lower() in ("n", "no"):
            self.logger.critical(f"{dataset_id} not deleted")
            return

        else:
            self.logger.critical(f"Error: Input {user_input} unrecognised.")
            return

    def clone(
        self,
        old_dataset: str,
        new_dataset: str,
        schema: dict = {},
        rename_fields: dict = {},
        remove_fields: list = [],
        filters: list = [],
        output_format: str = "json",
        verbose: bool = True,
    ):
        return self.make_http_request(
            endpoint=f"datasets/{old_dataset}/clone",
            method="POST",
            parameters={
                "new_dataset_id": new_dataset,
                "schema": schema,
                "rename_fields": rename_fields,
                "remove_fields": remove_fields,
                "filters": filters,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def search(
        self,
        query,
        sort_by_created_at_date: bool = False,
        asc: bool = False,
        output_format: str = "json",
        verbose: bool = True,
    ):
        return self.make_http_request(
            endpoint="datasets/search",
            method="GET",
            parameters={
                "query": query,
                "sort_by_created_at_date": sort_by_created_at_date,
                "asc": asc,
            },
            output_format=output_format,
            verbose=verbose,
        )



"""Chunk Helper functions
"""
from typing import List, Union

import pandas as pd

from ..progress_bar import progress_bar


class Chunker:
    """Update the chunk mixins"""

    def chunk(self, documents: Union[pd.DataFrame, List], chunksize: int = 20):
        """
        Chunk an iterable object in Python.
        Args:
            documents:
                List of dictionaries/Pandas dataframe
            chunksize:
                The chunk size of an object.
        Example:
            >>> documents = [{...}]
            >>> ViClient.chunk(documents)
        """
        if isinstance(documents, pd.DataFrame):
            for i in progress_bar(range(0, int(len(documents) / chunksize))):
                yield documents.iloc[i * chunksize : (i + 1) * chunksize]
        else:
            for i in progress_bar(range(0, int(len(documents) / chunksize))):
                yield documents[i * chunksize : ((i + 1) * chunksize)]



import os
import sys

from loguru import logger as loguru_logger

from .config import CONFIG
from .transport import Transport


def str2bool(v):
    return v.lower() in ("yes", "true", "t", "1")


class Base(Transport):
    """Base class for all relevanceai utilities"""

    def __init__(self, project: str, api_key: str, base_url: str):
        self.project = project
        self.api_key = api_key
        self.base_url = base_url
        self.config = CONFIG

    # Add Logging
    @property
    def logger(self):
        logging_level = self.config.get_option("logging.logging_level")
        log_to_file = str2bool(self.config.get_option("logging.log_to_file"))
        logger = loguru_logger
        logger.remove()
        logger.add(sys.stdout, level=logging_level)
        if log_to_file:
            logger.add(f"relevanceai.log", level=logging_level, rotation="100 MB")
        return logger



"""Recommmend services.
"""
from ..base import Base


class Recommend(Base):
    def __init__(self, project, api_key, base_url):
        self.project = project
        self.api_key = api_key
        self.base_url = base_url
        super().__init__(project, api_key, base_url)

    def vector(
        self,
        dataset_id: str,
        positive_document_ids: dict = {},
        negative_document_ids: dict = {},
        vector_fields=[],
        approximation_depth: int = 0,
        vector_operation: str = "sum",
        sum_fields: bool = True,
        page_size: int = 20,
        page: int = 1,
        similarity_metric: str = "cosine",
        facets: list = [],
        filters: list = [],
        min_score: float = 0,
        select_fields: list = [],
        include_vector: bool = False,
        include_count: bool = True,
        asc: bool = False,
        keep_search_history: bool = False,
    ):
        return self.make_http_request(
            f"services/recommend/recommend/vector",
            method="POST",
            parameters={
                "dataset_id": dataset_id,
                "positive_document_ids": positive_document_ids,
                "negative_document_ids": negative_document_ids,
                "vector_fields": vector_fields,
                "approximation_depth": approximation_depth,
                "vector_operation": vector_operation,
                "sum_fields": sum_fields,
                "page_size": page_size,
                "page": page,
                "similarity_metric": similarity_metric,
                "facets": facets,
                "filters": filters,
                "min_score": min_score,
                "select_fields": select_fields,
                "include_vector": include_vector,
                "include_count": include_count,
                "asc": asc,
                "keep_search_history": keep_search_history,
            },
        )



from ..base import Base


class Encoders(Base):
    def __init__(self, project: str, api_key: str, base_url: str):
        self.project = project
        self.api_key = api_key
        self.base_url = base_url
        super().__init__(project, api_key, base_url)

    def textimage(self, text: str):
        return self.make_http_request(
            "services/encoders/textimage", method="GET", parameters={"text": text}
        )

    def text(self, text: str):
        return self.make_http_request(
            "services/encoders/text", method="GET", parameters={"text": text}
        )

    def multi_text(self, text):
        """Encode Multilingual text"""
        return self.make_http_request(
            "services/encoders/multi_text", method="GET", parameters={"text": text}
        )



from ..base import Base


class Documents(Base):
    def __init__(self, project, api_key, base_url):
        self.project = project
        self.api_key = api_key
        self.base_url = base_url
        super().__init__(project, api_key, base_url)

    def list(
        self,
        dataset_id: str,
        cursor: str = None,
        page_size: int = 20,
        sort: list = [],
        include_vector: bool = True,
        random_state: int = 0,
        output_format: str = "json",
        verbose: bool = True,
    ):
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/documents/list",
            method="GET",
            parameters={
                "cursor": cursor,
                "page_size": page_size,
                "sort": sort,
                "include_vector": include_vector,
                "random_state": random_state,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def get(
        self,
        dataset_id: str,
        id: str,
        select_fields: list = [],
        cursor: str = None,
        page_size: int = 20,
        sort: list = [],
        include_vector: bool = True,
        output_format: str = "json",
        verbose: bool = True,
    ):
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/documents/get",
            parameters={
                "id": id,
                "select_fields": select_fields,
                "cursor": cursor,
                "page_size": page_size,
                "sort": sort,
                "include_vector": include_vector,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def get_where(
        self,
        dataset_id: str,
        filters: list = [],
        cursor: str = None,
        page_size: int = 20,
        sort: list = [],
        select_fields: list = [],
        include_vector: bool = True,
        random_state: int = 0,
        is_random: bool = False,
        output_format: str = "json",
        verbose: bool = True,
    ):
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/documents/get_where",
            method="POST",
            parameters={
                "select_fields": select_fields,
                "cursor": cursor,
                "page_size": page_size,
                "sort": sort,
                "include_vector": include_vector,
                "filters": filters,
                "random_state": random_state,
                "is_random": is_random,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def bulk_update(
        self,
        dataset_id: str,
        updates: list,
        output_format: str = "json",
        verbose: bool = True,
        return_documents: bool = False,
        retries=None,
    ):
        if return_documents is False:
            return self.make_http_request(
                endpoint=f"datasets/{dataset_id}/documents/bulk_update",
                method="POST",
                parameters={"updates": updates},
                output_format=output_format,
                verbose=verbose,
                retries=retries,
                base_url="https://ingest-api-dev-aueast.relevance.ai/latest/",
            )
        else:
            insert_response = self.make_http_request(
                endpoint=f"datasets/{dataset_id}/documents/bulk_update",
                method="POST",
                parameters={"updates": updates},
                output_format=False,
                verbose=verbose,
                retries=retries,
                base_url="https://ingest-api-dev-aueast.relevance.ai/latest/",
            )

            try:
                response_json = insert_response.json()
            except:
                response_json = None

            return {
                "response_json": response_json,
                "documents": updates,
                "status_code": insert_response.status_code,
            }

    def bulk_delete(
        self,
        dataset_id: str,
        ids: list = [],
        output_format: str = "json",
        verbose: bool = True,
    ):
        """Bulk delete."""
        return self.make_http_request(
            endpoint=f"datasets/{dataset_id}/documents/get_where",
            method="POST",
            parameters={"ids": ids},
            output_format=output_format,
            verbose=verbose,
        )

    def get_where_all(
        self,
        dataset_id: str,
        chunk_size: int = 10000,
        filters: list = [],
        sort: list = [],
        select_fields: list = [],
        include_vector: bool = True,
        random_state: int = 0,
        output_format: str = "json",
        verbose: bool = True,
    ):
        # Initialise values
        length = 1
        cursor = None
        full_data = []

        # While there is still data to fetch, fetch it at the latest cursor
        while length > 0:
            x = self.get_where(
                dataset_id,
                filters=filters,
                cursor=cursor,
                page_size=chunk_size,
                sort=sort,
                select_fields=select_fields,
                include_vector=include_vector,
                random_state=random_state,
                output_format=output_format,
                verbose=verbose,
            )
            length = len(x["documents"])
            cursor = x["cursor"]

            # Append fetched data to the full data
            if length > 0:
                [full_data.append(i) for i in x["documents"]]

        return full_data

    def get_number_of_documents(self, dataset_ids: list, list_of_filters=None):
        """
        dataset_ids: list of dataset_ids
        list_of_filters: list of list of filters corresponding to the same order of the dataset_ids
        """

        if list_of_filters is None:
            list_of_filters = [[] for _ in range(len(dataset_ids))]

        return {
            dataset_id: self._get_number_of_documents(dataset_id, filters)
            for dataset_id, filters in zip(dataset_ids, list_of_filters)
        }

    def _get_number_of_documents(self, dataset_id, filters=[]):
        """Certainty around the number of documents excluding chunks (until chunk documents is fixed)"""
        return self.get_where(dataset_id, page_size=1, filters=filters)["count"]



"""All Dataset related functions
"""
from ..base import Base


class Monitor(Base):
    def __init__(self, project, api_key, base_url):
        self.project = project
        self.api_key = api_key
        self.base_url = base_url
        super().__init__(project, api_key, base_url)

    def health(self, dataset_id: str):
        return self.make_http_request(
            f"datasets/{dataset_id}/monitor/health", method="GET", parameters={}
        )

    def stats(self, dataset_id: str):
        return self.make_http_request(
            f"datasets/{dataset_id}/monitor/stats", method="GET", parameters={}
        )



"""
Datasets to mock
"""
import warnings
from typing import List, Union

import pandas as pd
import requests


def get_games_dataset(number_of_documents: Union[None, int] = 365) -> List:
    """Function to download a sample games dataset.
    Dataset from https://www.freetogame.com/
    Total Len: 365
    Sample document:
    {'id': 1,
    'title': 'Dauntless',
    'thumbnail': 'https://www.freetogame.com/g/1/thumbnail.jpg',
    'short_description': 'A free-to-play, co-op action RPG with gameplay similar to Monster Hunter.',
    'game_url': 'https://www.freetogame.com/open/dauntless',
    'genre': 'MMORPG',
    'platform': 'PC (Windows)',
    'publisher': 'Phoenix Labs',
    'developer': 'Phoenix Labs, Iron Galaxy',
    'release_date': '2019-05-21',
    'freetogame_profile_url': 'https://www.freetogame.com/dauntless'
    }
    """
    data = requests.get("https://www.freetogame.com/api/games").json()
    if number_of_documents:
        data = data[:number_of_documents]
    return data


def get_dummy_ecommerce_dataset(
    db_name: str = "ecommerce-5",
    count: int = 1000,
    base_url="https://api-aueast.relevance.ai/v1/",
):
    """Here, we get the e-commerce dataset."""
    from .http_client import Client

    project = "dummy-collections"
    api_key = (
        "UzdYRktIY0JxNmlvb1NpOFNsenU6VGdTU0s4UjhUR0NsaDdnQTVwUkpKZw"  # read access
    )
    client = Client(project, api_key, base_url=base_url)
    response = client.datasets.documents.list(db_name, page_size=count)
    if "message" in response:
        warnings.warn(response["message"])
    return response

def get_sample_ecommerce_dataset(
    number_of_documents: int = 1000, vector_fields: list = ["product_image_clip_vector_"]
):
    """Here, we get the e-commerce dataset."""
    from .http_client import Client

    project = "dummy-collections"
    api_key = (
        "UzdYRktIY0JxNmlvb1NpOFNsenU6VGdTU0s4UjhUR0NsaDdnQTVwUkpKZw"  # read access
    )
    client = Client(
        project,
        api_key,
    )
    db_name = "quickstart_data_sample"
    response = client.datasets.documents.get_where(
        db_name,
        select_fields=["product_image", "product_title", "product_description"]
        + vector_fields,
        page_size=number_of_documents,
    )
    if "message" in response:
        import warnings

        warnings.warn(response["message"])
    docs = response["documents"]
    for d in docs:
        if "image_first" in d:
            d["image"] = d.pop("image_first")
    return docs


def get_online_retail_dataset(number_of_documents: Union[None, int] = 1000) -> List:
    """Online retail dataset from UCI machine learning
    Total Len: 406829
    Sample document:
    {'Country': 'United Kingdom',
     'CustomerID': 17850.0,
     'Description': 'WHITE HANGING HEART T-LIGHT HOLDER',
     'InvoiceDate': Timestamp('2010-12-01 08:26:00'),
     'InvoiceNo': 536365,
     'Quantity': 6,
     'StockCode': '85123A',
     'UnitPrice': 2.55}

    """
    return (
        pd.read_excel(
            "https://archive.ics.uci.edu/ml/machine-learning-databases/00352/Online%20Retail.xlsx"
        )
        .dropna()
        .iloc[:number_of_documents, :]
        .to_dict(orient="records")
    )


def get_news_dataset(number_of_documents: Union[None, int] = 250) -> List:
    """News dataset
    Total Len: 250
    Sample document:
    {'Unnamed: 0': 0,
     'authors': 'Ruth Harris',
     'content': 'Sometimes the power of Christmas will make you do wild and wonderful things. You do not need to believe in the Holy Trinity to believe in the positive power of doing good for others. The simple act of giving without receiving is lost on many of us these days, as worries about money and success hold us back from giving to others who are in need. One congregation in Ohio was moved to action by the power of a sermon given at their church on Christmas Eve. The pastor at Grand Lake United Methodist Church in Celina, Ohio gave an emotional sermon about the importance of understanding the message of Jesus.\n\nFor many religious people the message of Jesus is to help others before yourself, to make sure the people who are suffering get the help they need to enjoy life a little bit. The sermon was really about generosity and what that can look like in our lives. Jesus lived a long time ago and he acted generously in the fashion of his time – but what would a generous act look like in our times? That was the focus of the sermon.\n\nThe potency of the sermon was not lost on the congregation, who were so moved they had to take action! After the sermon ended, the congregation decided to take an offering. A bowl was passed around the room and everyone pitched in what they could on this Christmas Eve with the words of the sermon still ringing in their ears.\n\nWhat did they do with this offering? Members of the congregation drove down to the local Waffle House to visit the ladies working the night shift. What a great choice on this most holy of days when everyone should be with their families!\n\nThe ladies working at Waffle House clearly were not with their families. They had no choice but to work on this holy day because it paid the bills. The congregation understood the sacrifice being made by these ladies, and wanted to help them out. They donated the entire offering to be split amongst the ladies at Waffle House.\n\nIn total that amounted to $3,500 being split amongst the staff. What a beautiful moment! What a perfect example of what the preacher was talking about in his sermon! Doing a good deed like this on Christmas really helped ease the burden felt by the ladies working at Waffle House. Sure, they could not see their families, but at least they got a little gift from the good people of their community.\n\nPerhaps the best part about this whole event was that the congregation did not ask anything in return. It was a simple act of generosity from people who understood the pain being felt by another group and sought to alleviate some of that pain. It speaks volumes about the merits of the Church in our daily lives. This simple act brought the entire community together because it showed empathy and compassion on the most special day of the year.',
     'domain': 'awm.com',
     'id': 141,
     'inserted_at': '2018-02-02 01:19:41.756632',
     'keywords': nan,
     'meta_description': nan,
     'meta_keywords': "['']",
     'scraped_at': '2018-01-25 16:17:44.789555',
     'summary': nan,
     'tags': nan,
     'title': 'Church Congregation Brings Gift to Waitresses Working on Christmas Eve, Has Them Crying (video)',
     'type': 'unreliable',
     'updated_at': '2018-02-02 01:19:41.756664',
     'url': 'http://awm.com/church-congregation-brings-gift-to-waitresses-working-on-christmas-eve-has-them-crying-video/'}
    """
    return (
        pd.read_csv(
            "https://raw.githubusercontent.com/several27/FakeNewsCorpus/master/news_sample.csv"
        )
        .iloc[:number_of_documents, :]
        .to_dict(orient="records")
    )


def get_ecommerce_dataset(number_of_documents: Union[None, int] = 1000) -> List:
    """Function to download a sample ecommerce dataset
    Dataset from https://data.world/crowdflower/ecommerce-search-relevance
    Total Len: 15528
    Sample document:
    {'_unit_id': 711158459,
    'product_description': 'The PlayStation 4 system opens the door to an '
                        'incredible journey through immersive new gaming '
                        'worlds and a deeply connected gaming community. Step '
                        'into living, breathing worlds where you are hero of '
                        '...',
    'product_image': 'http://thumbs2.ebaystatic.com/d/l225/m/mzvzEUIknaQclZ801YCY1ew.jpg',
    'product_link': 'http://www.ebay.com/itm/Sony-PlayStation-4-PS4-Latest-Model-500-GB-Jet-Black-Console-/321459436277?pt=LH_DefaultDomain_0&hash=item4ad879baf5',
    'product_price': '$329.98 ',
    'product_title': 'Sony PlayStation 4 (PS4) (Latest Model)- 500 GB Jet Black '
                    'Console',
    'query': 'playstation 4',
    'rank': 1,
    'relevance': 3.67,
    'relevance:variance': 0.471,
    'source': 'eBay',
    'url': 'http://www.ebay.com/sch/i.html?_from=R40&_trksid=p2050601.m570.l1313.TR11.TRC1.A0.H0.Xplant.TRS0&_nkw=playstation%204'}
    """
    df = (
        pd.read_csv(
            "https://query.data.world/s/glc7oe2ssd252scha53mu7dy2e7cft",
            encoding="ISO-8859-1",
        )
        .dropna()
        .iloc[:number_of_documents, :]
    )
    df["product_image"] = df["product_image"].str.replace("http://", "https://")
    df["product_link"] = df["product_link"].str.replace("http://", "https://")
    df["url"] = df["url"].str.replace("http://", "https://")
    df["_id"] = df["_unit_id"].astype(str)
    return df.to_dict("records")



"""The Transport Class defines a transport as used by the Channel class to communicate with the network.
"""
import time
import traceback
from json.decoder import JSONDecodeError

import requests
from requests import Request

from .errors import APIError


class Transport:
    """Base class for all relevanceai objects"""

    project: str
    api_key: str

    @property
    def auth_header(self):
        return {"Authorization": self.project + ":" + self.api_key}

    def make_http_request(
        self,
        endpoint: str,
        method: str = "GET",
        parameters: dict = {},
        output_format: str = "json",
        base_url: str = None,
        verbose: bool = True,
        retries: bool = None,
    ):
        """Make the HTTP request
        Args:
            endpoint: The endpoint from the documentation to use
            method_type: POST or GET request
        """

        t1 = time.time()
        if base_url is None:
            base_url = self.base_url

        if retries is None:
            retries = int(self.config.get_option("retries.number_of_retries"))

        for _ in range(retries):
            if verbose:
                self.logger.info("URL you are trying to access:" + base_url + endpoint)
            try:
                req = Request(
                    method=method.upper(),
                    url=base_url + endpoint,
                    headers=self.auth_header,
                    json=parameters if method.upper() == "POST" else {},
                    params=parameters if method.upper() == "GET" else {},
                ).prepare()

                with requests.Session() as s:
                    response = s.send(req)

                if response.status_code == 200:
                    if verbose:
                        self.logger.success(
                            f"Response success! ({base_url + endpoint})"
                        )
                    time_diff = time.time() - t1
                    self.logger.debug(
                        f"Request ran in {time_diff} seconds ({base_url + endpoint})"
                    )

                    if output_format == "json":
                        return response.json()
                    else:
                        return response

                elif response.status_code == 404:
                    if verbose:
                        self.logger.error(response.content.decode())
                    if verbose:
                        self.logger.error(
                            f"Response failed ({base_url + endpoint}) (status: {response.status_code} Content: {response.content.decode()})"
                        )
                    raise APIError(response.content.decode())

                else:
                    if verbose:
                        self.logger.error(response.content.decode())
                    if verbose:
                        self.logger.error(
                            f"Response failed ({base_url + endpoint}) (status: {response.status_code} Content: {response.content.decode()})"
                        )
                    continue

            except (ConnectionError) as error:
                # Print the error
                traceback.print_exc()
                if verbose:
                    self.logger.error(
                        f"Connection error but re-trying. ({base_url + endpoint})"
                    )
                time.sleep(
                    int(self.config.get_option("retries.seconds_between_retries"))
                )
                continue

            except JSONDecodeError as error:
                if verbose:
                    self.logger.error(f"No Json available ({base_url + endpoint})")
                self.logger.error(response)

            if verbose:
                self.logger.error(
                    f"Response failed, stopped trying ({base_url + endpoint})"
                )
            raise APIError(response.content.decode())

        return response



"""access the client via this class
"""
import os

from doc_utils import DocUtils

from .batch.client import BatchAPIClient
from .errors import APIError


def str2bool(v):
    return v.lower() in ("yes", "true", "t", "1")


class Client(BatchAPIClient, DocUtils):
    """Python Client for Relevance AI's relevanceai"""

    WELCOME_MESSAGE = """Welcome to the development version of the relevanceai Python SDK"""
    FAIL_MESSAGE = """Your API key is invalid. Please login again"""

    def __init__(
        self,
        project: str = os.getenv("VDB_PROJECT"),
        api_key: str = os.getenv("VDB_API_KEY"),
        base_url: str = "https://gateway-api-aueast.relevance.ai/v1/",
    ):
        super().__init__(project, api_key, base_url)

        if project is None or api_key is None:
            raise ValueError(
                "It seems you are missing an API key, "
                + "you can sign up for an API key following the instructions here: "
                + "https://discovery.relevance.ai/reference/usage"
            )

        if (
            self.datasets.list(
                verbose=False, output_format=False, retries=1
            ).status_code
            == 200
        ):
            self.logger.success(self.WELCOME_MESSAGE)
        else:
            raise APIError(self.FAIL_MESSAGE)

    @property
    def auth_header(self):
        return {"Authorization": self.project + ":" + self.api_key}



from ..base import Base


class Search(Base):
    def __init__(self, project, api_key, base_url):
        self.project = project
        self.api_key = api_key
        self.base_url = base_url
        super().__init__(project, api_key, base_url)

    def vector(
        self,
        dataset_id: str,
        multivector_query: list,
        positive_document_ids: dict = {},
        negative_document_ids: dict = {},
        vector_operation="sum",
        approximation_depth=0,
        sum_fields=True,
        page_size=20,
        page=1,
        similarity_metric="cosine",
        facets=[],
        filters=[],
        min_score=0,
        select_fields=[],
        include_vector=False,
        include_count=True,
        asc=False,
        keep_search_history=False,
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/vector",
            method="POST",
            parameters={
                "dataset_id": dataset_id,
                "multivector_query": multivector_query,
                "positive_document_ids": positive_document_ids,
                "negative_document_ids": negative_document_ids,
                "vector_operation": vector_operation,
                "approximation_depth": approximation_depth,
                "sum_fields": sum_fields,
                "page_size": page_size,
                "page": page,
                "similarity_metric": similarity_metric,
                "facets": facets,
                "filters": filters,
                "min_score": min_score,
                "select_fields": select_fields,
                "include_vector": include_vector,
                "include_count": include_count,
                "asc": asc,
                "keep_search_history": keep_search_history,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def hybrid(
        self,
        dataset_id: str,
        multivector_query: list,
        text: str,
        fields: list,
        edit_distance: int = -1,
        ignore_spaces: bool = True,
        traditional_weight: float = 0.075,
        page_size: int = 20,
        page=1,
        similarity_metric="cosine",
        facets=[],
        filters=[],
        min_score=0,
        select_fields=[],
        include_vector=False,
        include_count=True,
        asc=False,
        keep_search_history=False,
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/hybrid",
            method="POST",
            parameters={
                "dataset_id": "ecommerce-experiments",
                "dataset_id": dataset_id,
                "multivector_query": multivector_query,
                "text": text,
                "fields": fields,
                "page_size": page_size,
                "page": page,
                "similarity_metric": similarity_metric,
                "facets": facets,
                "filters": filters,
                "min_score": min_score,
                "select_fields": select_fields,
                "include_vector": include_vector,
                "include_count": include_count,
                "asc": asc,
                "keep_search_history": keep_search_history,
                "edit_distance": edit_distance,
                "ignore_spaces": ignore_spaces,
                "traditional_weight": traditional_weight,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def semantic(
        self,
        dataset_id: str,
        multivector_query: list,
        fields: list,
        text: str,
        page_size: int = 20,
        page=1,
        similarity_metric="cosine",
        facets=[],
        filters=[],
        min_score=0,
        select_fields=[],
        include_vector=False,
        include_count=True,
        asc=False,
        keep_search_history=False,
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/semantic",
            method="POST",
            parameters={
                "dataset_id": dataset_id,
                "multivector_query": multivector_query,
                "text": text,
                "fields": fields,
                "page_size": page_size,
                "page": page,
                "similarity_metric": similarity_metric,
                "facets": facets,
                "filters": filters,
                "min_score": min_score,
                "select_fields": select_fields,
                "include_vector": include_vector,
                "include_count": include_count,
                "asc": asc,
                "keep_search_history": keep_search_history,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def diversity(
        self,
        dataset_id: str,
        cluster_vector_field: str,
        multivector_query: list,
        positive_document_ids: dict = {},
        negative_document_ids: dict = {},
        vector_operation="sum",
        approximation_depth: int = 0,
        sum_fields: bool = True,
        page_size: int = 20,
        page: int = 1,
        similarity_metric="cosine",
        facets=[],
        filters=[],
        min_score=0,
        select_fields=[],
        include_vector=False,
        include_count=True,
        asc=False,
        keep_search_history=False,
        hundred_scale: bool = False,
        search_history_id: str = None,
        n_clusters: int = 0,
        n_init: int = 5,
        n_iter: int = 10,
        return_as_clusters: bool = False,
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/diversity",
            method="POST",
            parameters={
                "dataset_id": dataset_id,
                "multivector_query": multivector_query,
                "positive_document_ids": positive_document_ids,
                "negative_document_ids": negative_document_ids,
                "vector_operation": vector_operation,
                "approximation_depth": approximation_depth,
                "sum_fields": sum_fields,
                "page_size": page_size,
                "page": page,
                "similarity_metric": similarity_metric,
                "facets": facets,
                "filters": filters,
                "min_score": min_score,
                "select_fields": select_fields,
                "include_vector": include_vector,
                "include_count": include_count,
                "asc": asc,
                "keep_search_history": keep_search_history,
                "hundred_scale": hundred_scale,
                "search_history_id": search_history_id,
                "cluster_vector_field": cluster_vector_field,
                "n_clusters": n_clusters,
                "n_init": n_init,
                "n_iter": n_iter,
                "return_as_clusters": return_as_clusters,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def traditional(
        self,
        dataset_id: str,
        text: str,
        fields: list = [],
        edit_distance: int = -1,
        ignore_spaces: bool = True,
        page_size: int = 29,
        page: int = 1,
        select_fields: list = [],
        include_vector: bool = False,
        include_count: bool = True,
        asc: bool = False,
        keep_search_history: bool = False,
        search_history_id: str = None,
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/traditional",
            method="POST",
            parameters={
                "dataset_id": dataset_id,
                "text": text,
                "fields": fields,
                "edit_distance": edit_distance,
                "ignore_spaces": ignore_spaces,
                "page_size": page_size,
                "page": page,
                "select_fields": select_fields,
                "include_vector": include_vector,
                "include_count": include_count,
                "asc": asc,
                "keep_search_history": keep_search_history,
                "search_history_id": search_history_id,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def chunk(
        self,
        dataset_id,
        multivector_query,
        chunk_field,
        chunk_scoring="max",
        chunk_page_size: int = 3,
        chunk_page: int = 1,
        approximation_depth: int = 0,
        sum_fields: bool = True,
        page_size: int = 20,
        page: int = 1,
        similarity_metric: str = "cosine",
        facets: list = [],
        filters: list = [],
        min_score: int = None,
        include_vector: bool = False,
        include_count: bool = True,
        asc: bool = False,
        keep_search_history: bool = False,
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/chunk",
            method="POST",
            parameters={
                "dataset_id": dataset_id,
                "multivector_query": multivector_query,
                "chunk_field": chunk_field,
                "chunk_scoring": chunk_scoring,
                "chunk_page_size": chunk_page_size,
                "chunk_page": chunk_page,
                "approximation_depth": approximation_depth,
                "sum_fields": sum_fields,
                "page_size": page_size,
                "page": page,
                "similarity_metric": similarity_metric,
                "facets": facets,
                "filters": filters,
                "min_score": min_score,
                "include_vector": include_vector,
                "include_count": include_count,
                "asc": asc,
                "keep_search_history": keep_search_history,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def multistep_chunk(
        self,
        dataset_id,
        multivector_query,
        first_step_multivector_query,
        chunk_field,
        chunk_scoring="max",
        chunk_page_size: int = 3,
        chunk_page: int = 1,
        approximation_depth: int = 0,
        sum_fields: bool = True,
        page_size: int = 20,
        page: int = 1,
        similarity_metric: str = "cosine",
        facets: list = [],
        filters: list = [],
        min_score: int = None,
        include_vector: bool = False,
        include_count: bool = True,
        asc: bool = False,
        keep_search_history: bool = False,
        first_step_page: int = 1,
        first_step_page_size: int = 20,
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/multistep_chunk",
            method="POST",
            parameters={
                "dataset_id": dataset_id,
                "multivector_query": multivector_query,
                "chunk_field": chunk_field,
                "chunk_scoring": chunk_scoring,
                "chunk_page_size": chunk_page_size,
                "chunk_page": chunk_page,
                "approximation_depth": approximation_depth,
                "sum_fields": sum_fields,
                "page_size": page_size,
                "page": page,
                "similarity_metric": similarity_metric,
                "facets": facets,
                "filters": filters,
                "min_score": min_score,
                "include_vector": include_vector,
                "include_count": include_count,
                "asc": asc,
                "keep_search_history": keep_search_history,
                "first_step_multivector_query": first_step_multivector_query,
                "first_step_page": first_step_page,
                "first_step_page_size": first_step_page_size,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def advanced_chunk(
        self,
        dataset_ids,
        chunk_search_query,
        min_score: int = None,
        page_size: int = 20,
        include_vector: bool = False,
        select_fields: list = [],
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/advanced_chunk",
            method="POST",
            parameters={
                "dataset_ids": dataset_ids,
                "chunk_search_query": chunk_search_query,
                "page_size": page_size,
                "min_score": min_score,
                "include_vector": include_vector,
                "select_fields": select_fields,
            },
            output_format=output_format,
            verbose=verbose,
        )

    def advanced_multistep_chunk(
        self,
        dataset_ids: list,
        first_step_query: list,
        first_step_text: str,
        first_step_fields: list,
        chunk_search_query: list,
        first_step_edit_distance: int = -1,
        first_step_ignore_space: bool = True,
        first_step_traditional_weight: float = 0.075,
        first_step_approximation_depth: int = 0,
        first_step_sum_fields: bool = True,
        first_step_filters: list = [],
        first_step_page_size: int = 50,
        include_count: bool = True,
        min_score: int = 0,
        page_size: int = 20,
        include_vector: bool = False,
        select_fields: list = [],
        verbose: bool = True,
        output_format: str = "json",
    ):
        return self.make_http_request(
            "services/search/advanced_multistep_chunk",
            method="POST",
            parameters={
                "dataset_ids": dataset_ids,
                "first_step_query": first_step_query,
                "first_step_text": first_step_text,
                "first_step_fields": first_step_fields,
                "chunk_search_query": chunk_search_query,
                "first_step_edit_distance": first_step_edit_distance,
                "first_step_ignore_space": first_step_ignore_space,
                "first_step_traditional_weight": first_step_traditional_weight,
                "first_step_approximation_depth": first_step_approximation_depth,
                "first_step_sum_fields": first_step_sum_fields,
                "first_step_filters": first_step_filters,
                "first_step_page_size": first_step_page_size,
                "include_count": include_count,
                "min_score": min_score,
                "page_size": page_size,
                "include_vector": include_vector,
                "select_fields": select_fields,
            },
            output_format=output_format,
            verbose=verbose,
        )



import os

from setuptools import find_packages, setup


def read(rel_path):
    """Read lines from given file"""
    here = os.path.abspath(os.path.dirname(__file__))
    with open(os.path.join(here, rel_path), "r") as fp:
        return fp.read()


def get_version(rel_path):
    """Read __version__ from given file"""
    for line in read(rel_path).splitlines():
        if line.startswith("__version__"):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    raise RuntimeError(f"Unable to find a valid __version__ string in {rel_path}.")


requirements = [
    "tqdm>=4.49.0",
    "pandas>=1.0.0",
    "loguru>=0.5.3",
    "document-utils>=1.3.0",
    "requests>=2.0.0",
    "numpy>=1.19.0",
]

dev_requirements = [
    "autopep8",
    "pylint",
    "pytest",
    "pytest-dotenv",
    "pytest-cov",
    "pytest-mock",
]

setup(
    name="RelevanceAI",
    version=get_version("relevanceai/__init__.py"),
    url="",
    author="Relevance AI",
    author_email="dev@relevance.ai",
    long_description="",
    packages=find_packages(),
    install_requires=requirements,
    package_data={
        "": [
            "*.ini",
        ]
    },
    extras_require={
        "dev": dev_requirements,
        "excel": ["fsspec==2021.10.1", "openpyxl==3.0.9"],
        "tests": ["pytest", "fsspec==2021.10.1", "openpyxl==3.0.9"],
        "notebook": ["jsonshower"],
    },
    # python_requires=">=3.7",
    classifiers=[],
)



"""API Client
"""
from loguru import logger as loguru_logger

from ..base import Base
from .admin import Admin
from .datasets import Datasets
from .services import Services


def str2bool(v):
    return v.lower() in ("yes", "true", "t", "1")


class APIClient(Base):
    """API Client"""

    def __init__(self, project: str, api_key: str, base_url: str):
        self.datasets = Datasets(project=project, api_key=api_key, base_url=base_url)
        self.services = Services(project=project, api_key=api_key, base_url=base_url)
        self.admin = Admin(project=project, api_key=api_key, base_url=base_url)
        super().__init__(project, api_key, base_url)



import configparser
import os

from doc_utils.doc_utils import DocUtils

PATH = os.path.dirname(os.path.abspath(__file__))
CONFIG_PATH = os.path.join(PATH, "config.ini")


class Config(DocUtils):
    def __init__(self):
        self.config = configparser.ConfigParser()
        self.read_config(CONFIG_PATH)
        super().__init__()

    def read_config(self, config_path):
        self.config.read(config_path)

    def view_options(self):
        return self.config._sections

    def get_option(self, option):
        return self.get_field(option, self.config)

    def set_option(self, option, value):
        self.set_field(option, self.config, str(value))


CONFIG = Config()

if __name__ == "__main__":
    config = configparser.ConfigParser()
    config["retries"] = {"number_of_retries": 1, "seconds_between_retries": 2}
    config["logging"] = {"log_to_file": False, "logging_level": "SUCCESS"}
    config["upload"] = {"target_chunk_mb": 100}
    with open(CONFIG_PATH, "w") as configfile:
        config.write(configfile)
    print(CONFIG_PATH)
